#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
import datetime
import errno
import json
import logging
import os
import shutil
import sys
import tempfile
import time
import uuid

from osc_lib import exceptions as oscexc
from osc_lib.i18n import _
import six
from six.moves.urllib import parse
import yaml

from tripleo_common.image.builder import buildah
from tripleo_common.image import image_uploader
from tripleo_common.image import kolla_builder
from tripleo_common.utils.locks import processlock
from tripleoclient import utils as oooutils

from tripleoclient import command
from tripleoclient import constants
from tripleoclient import exceptions
from tripleoclient import utils


def build_env_file(params, command_options):

    f = six.StringIO()
    f.write('# Generated with the following on %s\n#\n' %
            datetime.datetime.now().isoformat())
    f.write('#   openstack %s\n#\n\n' %
            ' '.join(command_options))

    yaml.safe_dump({'parameter_defaults': params}, f,
                   default_flow_style=False)
    return f.getvalue()


class UploadImage(command.Command):
    """Push overcloud container images to registries."""

    auth_required = False
    log = logging.getLogger(__name__ + ".UploadImage")

    def get_parser(self, prog_name):
        parser = super(UploadImage, self).get_parser(prog_name)
        parser.add_argument(
            "--config-file",
            dest="config_files",
            metavar='<yaml config file>',
            default=[],
            action="append",
            required=True,
            help=_("YAML config file specifying the image build. May be "
                   "specified multiple times. Order is preserved, and later "
                   "files will override some options in previous files. "
                   "Other options will append."),
        )
        parser.add_argument(
            "--cleanup",
            dest="cleanup",
            metavar='<full, partial, none>',
            default=image_uploader.CLEANUP_FULL,
            help=_("Cleanup behavior for local images left after upload. "
                   "The default 'full' will attempt to delete all local "
                   "images. 'partial' will leave images required for "
                   "deployment on this host. 'none' will do no cleanup.")
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        if parsed_args.cleanup not in image_uploader.CLEANUP:
            raise oscexc.CommandError('--cleanup must be one of: %s' %
                                      ', '.join(image_uploader.CLEANUP))
        lock = processlock.ProcessLock()
        uploader = image_uploader.ImageUploadManager(
            parsed_args.config_files, cleanup=parsed_args.cleanup, lock=lock)
        try:
            uploader.upload()
        except KeyboardInterrupt:  # ctrl-c
            self.log.warning('Upload was interrupted by ctrl-c.')


class BuildImage(command.Command):
    """Build overcloud container images with kolla-build."""

    auth_required = False
    log = logging.getLogger(__name__ + ".BuildImage")

    @staticmethod
    def images_from_deps(images, dep):
        '''Builds a list from the dependencies depth-first. '''
        if isinstance(dep, list):
            for v in dep:
                BuildImage.images_from_deps(images, v)
        elif isinstance(dep, dict):
            for k, v in dep.items():
                images.append(k)
                BuildImage.images_from_deps(images, v)
        else:
            images.append(dep)

    def get_parser(self, prog_name):
        default_kolla_conf = os.path.join(
            sys.prefix, 'share', 'tripleo-common', 'container-images',
            'tripleo_kolla_config_overrides.conf')
        parser = super(BuildImage, self).get_parser(prog_name)
        parser.add_argument(
            "--config-file",
            dest="config_files",
            metavar='<yaml config file>',
            default=[],
            action="append",
            help=_("YAML config file specifying the images to build. May be "
                   "specified multiple times. Order is preserved, and later "
                   "files will override some options in previous files. "
                   "Other options will append. If not specified, the default "
                   "set of containers will be built."),
        )
        parser.add_argument(
            "--kolla-config-file",
            dest="kolla_config_files",
            metavar='<config file>',
            default=[default_kolla_conf],
            action="append",
            required=True,
            help=_("Path to a Kolla config file to use. Multiple config files "
                   "can be specified, with values in later files taking "
                   "precedence. By default, tripleo kolla conf file {conf} "
                   "is added.").format(conf=default_kolla_conf),
        )
        parser.add_argument(
            '--list-images',
            dest='list_images',
            action='store_true',
            default=False,
            help=_('Show the images which would be built instead of '
                   'building them.')
        )
        parser.add_argument(
            '--list-dependencies',
            dest='list_dependencies',
            action='store_true',
            default=False,
            help=_('Show the image build dependencies instead of '
                   'building them.')
        )
        parser.add_argument(
            "--exclude",
            dest="excludes",
            metavar='<container-name>',
            default=[],
            action="append",
            help=_("Name of a container to match against the list of "
                   "containers to be built to skip. Can be specified multiple "
                   "times."),
        )
        parser.add_argument(
            '--use-buildah',
            dest='use_buildah',
            action='store_true',
            default=False,
            help=_('Use Buildah instead of Docker to build the images '
                   'with Kolla.')
        )
        parser.add_argument(
            "--work-dir",
            dest="work_dir",
            default='/tmp/container-builds',
            metavar='<container builds directory>',
            help=_("TripleO container builds directory, storing configs and "
                   "logs for each image and its dependencies.")
        )
        parser.add_argument(
            "--build-timeout",
            dest="build_timeout",
            default=None,
            type=int,
            metavar="<build timeout in seconds>",
            help=_("Build timeout in seconds.")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        self.log.warning('This command is deprecated. Please use "openstack '
                         'tripleo container image build" instead.')

        fd, path = tempfile.mkstemp(prefix='kolla_conf_')
        with os.fdopen(fd, 'w') as tmp:
            tmp.write('[DEFAULT]\n')
            if parsed_args.list_images or parsed_args.list_dependencies:
                tmp.write('list_dependencies=true')
        kolla_config_files = list(parsed_args.kolla_config_files)
        kolla_config_files.append(path)
        # Generate an unique work directory so we can keep configs and logs
        # each time we run the command; they'll be stored in work_dir.
        kolla_work_dir = os.path.join(parsed_args.work_dir, str(uuid.uuid4()))

        # Make sure the unique work directory exists
        if not os.path.exists(kolla_work_dir):
            self.log.debug("Creating container builds "
                           "workspace in: %s" % kolla_work_dir)
            os.makedirs(kolla_work_dir)

        builder = kolla_builder.KollaImageBuilder(parsed_args.config_files)
        result = builder.build_images(kolla_config_files,
                                      parsed_args.excludes,
                                      parsed_args.use_buildah,
                                      kolla_work_dir)

        if parsed_args.use_buildah:
            deps = json.loads(result)
            kolla_cfg = utils.get_read_config(kolla_config_files)
            bb = buildah.BuildahBuilder(
                kolla_work_dir, deps,
                utils.get_from_cfg(kolla_cfg, "base"),
                utils.get_from_cfg(kolla_cfg, "type"),
                utils.get_from_cfg(kolla_cfg, "tag"),
                utils.get_from_cfg(kolla_cfg, "namespace"),
                utils.get_from_cfg(kolla_cfg, "registry"),
                utils.getboolean_from_cfg(kolla_cfg, "push"),
                parsed_args.build_timeout,
                self.app.options.debug)
            bb.build_all()
        elif parsed_args.list_dependencies:
            deps = json.loads(result)
            yaml.safe_dump(
                deps,
                self.app.stdout,
                indent=2,
                default_flow_style=False
            )
        elif parsed_args.list_images:
            deps = json.loads(result)
            images = []
            BuildImage.images_from_deps(images, deps)
            yaml.safe_dump(
                images,
                self.app.stdout,
                default_flow_style=False
            )
        elif result:
            self.app.stdout.write(result)


class PrepareImageFiles(command.Command):
    """Generate files defining the images, tags and registry."""

    auth_required = False
    log = logging.getLogger(__name__ + ".PrepareImageFiles")

    def get_parser(self, prog_name):
        parser = super(PrepareImageFiles, self).get_parser(prog_name)
        try:
            roles_file = utils.rel_or_abs_path(
                constants.OVERCLOUD_ROLES_FILE,
                constants.TRIPLEO_HEAT_TEMPLATES)
        except exceptions.DeploymentError:
            roles_file = None
        defaults = kolla_builder.container_images_prepare_defaults()

        parser.add_argument(
            "--template-file",
            dest="template_file",
            default=kolla_builder.DEFAULT_TEMPLATE_FILE,
            metavar='<yaml template file>',
            help=_("YAML template file which the images config file will be "
                   "built from.\n"
                   "Default: %s") % kolla_builder.DEFAULT_TEMPLATE_FILE,
        )
        parser.add_argument(
            "--push-destination",
            dest="push_destination",
            metavar='<location>',
            help=_("Location of image registry to push images to. "
                   "If specified, a push_destination will be set for every "
                   "image entry."),
        )
        parser.add_argument(
            "--tag",
            dest="tag",
            default=defaults['tag'],
            metavar='<tag>',
            help=_("Override the default tag substitution. "
                   "If --tag-from-label is specified, "
                   "start discovery with this tag.\n"
                   "Default: %s") % defaults['tag'],
        )
        parser.add_argument(
            "--tag-from-label",
            dest="tag_from_label",
            metavar='<image label>',
            help=_("Use the value of the specified label(s) to discover the "
                   "tag. Labels can be combined in a template format, "
                   "for example: {version}-{release}"),
        )
        parser.add_argument(
            "--namespace",
            dest="namespace",
            default=defaults['namespace'],
            metavar='<namespace>',
            help=_("Override the default namespace substitution.\n"
                   "Default: %s") % defaults['namespace'],
        )
        parser.add_argument(
            "--prefix",
            dest="prefix",
            default=defaults['name_prefix'],
            metavar='<prefix>',
            help=_("Override the default name prefix substitution.\n"
                   "Default: %s") % defaults['name_prefix'],
        )
        parser.add_argument(
            "--suffix",
            dest="suffix",
            default=defaults['name_suffix'],
            metavar='<suffix>',
            help=_("Override the default name suffix substitution.\n"
                   "Default: %s") % defaults['name_suffix'],
        )
        parser.add_argument(
            '--set',
            metavar='<variable=value>',
            action='append',
            help=_('Set the value of a variable in the template, even if it '
                   'has no dedicated argument such as "--suffix".')
        )
        parser.add_argument(
            "--exclude",
            dest="excludes",
            metavar='<regex>',
            default=[],
            action="append",
            help=_("Pattern to match against resulting imagename entries to "
                   "exclude from the final output. Can be specified multiple "
                   "times."),
        )
        parser.add_argument(
            "--include",
            dest="includes",
            metavar='<regex>',
            default=[],
            action="append",
            help=_("Pattern to match against resulting imagename entries to "
                   "include in final output. Can be specified multiple "
                   "times, entries not matching any --include will be "
                   "excluded. --exclude is ignored if --include is used."),
        )
        parser.add_argument(
            "--output-images-file",
            dest="output_images_file",
            metavar='<file path>',
            help=_("File to write resulting image entries to, as well as "
                   "stdout. Any existing file will be overwritten."),
        )
        parser.add_argument(
            '--environment-file', '-e', metavar='<file path>',
            action='append', dest='environment_files',
            help=_('Environment files specifying which services are '
                   'containerized. Entries will be filtered to only contain '
                   'images used by containerized services. (Can be specified '
                   'more than once.)')
        )
        parser.add_argument(
            '--environment-directory', metavar='<HEAT ENVIRONMENT DIRECTORY>',
            action='append', dest='environment_directories',
            default=[os.path.expanduser(constants.DEFAULT_ENV_DIRECTORY)],
            help=_('Environment file directories that are automatically '
                   'added to the update command. Entries will be filtered '
                   'to only contain images used by containerized services. '
                   'Can be specified more than once. Files in directories are '
                   'loaded in ascending sort order.')
        )
        parser.add_argument(
            "--output-env-file",
            dest="output_env_file",
            metavar='<file path>',
            help=_("File to write heat environment file which specifies all "
                   "image parameters. Any existing file will be overwritten."),
        )
        parser.add_argument(
            '--roles-file', '-r', dest='roles_file',
            default=roles_file,
            help=_(
                'Roles file, overrides the default %s in the t-h-t templates '
                'directory used for deployment. May be an '
                'absolute path or the path relative to the templates dir.'
                ) % constants.OVERCLOUD_ROLES_FILE
        )
        parser.add_argument(
            '--modify-role',
            dest='modify_role',
            help=_('Name of ansible role to run between every image upload '
                   'pull and push.')
        )
        parser.add_argument(
            '--modify-vars',
            dest='modify_vars',
            help=_('Ansible variable file containing variables to use when '
                   'invoking the role --modify-role.')
        )
        return parser

    def parse_set_values(self, subs, set_values):
        if not set_values:
            return
        for s in set_values:
            try:
                (n, v) = s.split(('='), 1)
                subs[n] = v
            except ValueError:
                msg = _('Malformed --set(%s). '
                        'Use the variable=value format.') % s
                raise oscexc.CommandError(msg)

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        self.log.warning("[DEPRECATED] This command has been deprecated and "
                         "replaced by the 'openstack tripleo container image "
                         "prepare' command.")

        roles_data = utils.fetch_roles_file(parsed_args.roles_file) or set()

        env = utils.build_prepare_env(
            parsed_args.environment_files,
            parsed_args.environment_directories
        )

        if roles_data:
            service_filter = kolla_builder.build_service_filter(
                env, roles_data)
        else:
            service_filter = None
        mapping_args = {
            'tag': parsed_args.tag,
            'namespace': parsed_args.namespace,
            'name_prefix': parsed_args.prefix,
            'name_suffix': parsed_args.suffix,
        }
        self.parse_set_values(mapping_args, parsed_args.set)
        pd = env.get('parameter_defaults', {})
        kolla_builder.set_neutron_driver(pd, mapping_args)

        output_images_file = (parsed_args.output_images_file
                              or 'container_images.yaml')
        modify_role = None
        modify_vars = None
        append_tag = None
        if parsed_args.modify_role:
            modify_role = parsed_args.modify_role
            append_tag = time.strftime('-modified-%Y%m%d%H%M%S')
        if parsed_args.modify_vars:
            with open(parsed_args.modify_vars) as m:
                modify_vars = yaml.safe_load(m.read())

        prepare_data = kolla_builder.container_images_prepare(
            excludes=parsed_args.excludes,
            includes=parsed_args.includes,
            service_filter=service_filter,
            push_destination=parsed_args.push_destination,
            mapping_args=mapping_args,
            output_env_file=parsed_args.output_env_file,
            output_images_file=output_images_file,
            tag_from_label=parsed_args.tag_from_label,
            modify_role=modify_role,
            modify_vars=modify_vars,
            append_tag=append_tag,
            template_file=parsed_args.template_file
        )
        if parsed_args.output_env_file:
            params = prepare_data[parsed_args.output_env_file]
            output_env_file_expanded = os.path.expanduser(
                parsed_args.output_env_file)
            if os.path.exists(output_env_file_expanded):
                self.log.warning("Output env file exists, "
                                 "moving it to backup.")
                shutil.move(output_env_file_expanded,
                            output_env_file_expanded + ".backup")
            utils.safe_write(output_env_file_expanded,
                             build_env_file(params, self.app.command_options))

        result = prepare_data[output_images_file]
        result_str = yaml.safe_dump({'container_images': result},
                                    default_flow_style=False)
        sys.stdout.write(result_str)

        if parsed_args.output_images_file:
            utils.safe_write(parsed_args.output_images_file, result_str)


class DiscoverImageTag(command.Command):
    """Discover the versioned tag for an image."""

    auth_required = False
    log = logging.getLogger(__name__ + ".DiscoverImageTag")

    def get_parser(self, prog_name):
        parser = super(DiscoverImageTag, self).get_parser(prog_name)
        parser.add_argument(
            "--image",
            dest="image",
            metavar='<container image>',
            required=True,
            help=_("Fully qualified name of the image to discover the tag for "
                   "(Including registry and stable tag)."),
        )
        parser.add_argument(
            "--tag-from-label",
            dest="tag_from_label",
            metavar='<image label>',
            help=_("Use the value of the specified label(s) to discover the "
                   "tag. Labels can be combined in a template format, "
                   "for example: {version}-{release}"),
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)
        self.log.warning("[DEPRECATED] This command has been deprecated and "
                         "replaced by the 'openstack tripleo container image "
                         "prepare' command.")

        lock = processlock.ProcessLock()
        uploader = image_uploader.ImageUploadManager([], lock=lock)
        print(uploader.discover_image_tag(
            image=parsed_args.image,
            tag_from_label=parsed_args.tag_from_label
        ))


class TripleOContainerImagePush(command.Command):
    """Push specified image to registry."""

    auth_required = False
    log = logging.getLogger(__name__ + ".TripleoContainerImagePush")

    def get_parser(self, prog_name):
        parser = super(TripleOContainerImagePush, self).get_parser(prog_name)
        parser.add_argument(
            "--local",
            dest="local",
            default=False,
            action="store_true",
            help=_("Use this flag if the container image is already on the "
                   "current system and does not need to be pulled from a "
                   "remote registry.")
        )
        parser.add_argument(
            "--registry-url",
            dest="registry_url",
            metavar='<registry url>',
            default=None,
            help=_("URL of the destination registry in the form "
                   "<fqdn>:<port>.")
        )
        parser.add_argument(
            "--append-tag",
            dest="append_tag",
            default='',
            help=_("Tag to append to the existing tag when pushing the "
                   "container. ")
        )
        parser.add_argument(
            "--username",
            dest="username",
            metavar='<username>',
            help=_("Username for the destination image registry.")
        )
        parser.add_argument(
            "--password",
            dest="password",
            metavar='<password>',
            help=_("Password for the destination image registry.")
        )
        parser.add_argument(
            "--source-username",
            dest="source_username",
            metavar='<source_username>',
            help=_("Username for the source image registry.")
        )
        parser.add_argument(
            "--source-password",
            dest="source_password",
            metavar='<source_password>',
            help=_("Password for the source image registry.")
        )

        parser.add_argument(
            "--dry-run",
            dest="dry_run",
            action="store_true",
            help=_("Perform a dry run upload. The upload action is not "
                   "performed, but the authentication process is attempted.")
        )
        parser.add_argument(
            "--multi-arch",
            dest="multi_arch",
            action="store_true",
            help=_("Enable multi arch support for the upload.")
        )
        parser.add_argument(
            "--cleanup",
            dest="cleanup",
            action="store_true",
            default=False,
            help=_("Remove local copy of the image after uploading")
        )
        parser.add_argument(
            dest="image_to_push",
            metavar='<image to push>',
            help=_("Container image to upload. Should be in the form of "
                   "<registry>/<namespace>/<name>:<tag>. If tag is "
                   "not provided, then latest will be used.")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)

        lock = processlock.ProcessLock()
        manager = image_uploader.ImageUploadManager(lock=lock)
        uploader = manager.uploader('python')

        source_image = parsed_args.image_to_push

        if parsed_args.local or source_image.startswith('containers-storage:'):
            storage = 'containers-storage:'
            if not source_image.startswith(storage):
                source_image = storage + source_image.replace('docker://', '')
            elif not parsed_args.local:
                self.log.warning('Assuming local container based on provided '
                                 'container path. (e.g. starts with '
                                 'containers-storage:)')
            source_url = parse.urlparse(source_image)
            image_name = source_url.geturl()
            image_source = None
            if parsed_args.source_username or parsed_args.source_password:
                self.log.warning('Source credentials ignored for local images')
        else:
            storage = 'docker://'
            if not source_image.startswith(storage):
                source_image = storage + source_image
            source_url = parse.urlparse(source_image)
            image_source = source_url.netloc
            image_name = source_url.path[1:]
            if len(image_name.split('/')) != 2:
                raise exceptions.DownloadError('Invalid container. Provided '
                                               'container image should be '
                                               '<registry>/<namespace>/<name>:'
                                               '<tag>')
            if parsed_args.source_username or parsed_args.source_password:
                if not parsed_args.source_username:
                    self.log.warning('Skipping authentication - missing source'
                                     ' username')
                elif not parsed_args.source_password:
                    self.log.warning('Skipping authentication - missing source'
                                     ' password')
                else:
                    uploader.authenticate(source_url,
                                          parsed_args.source_username,
                                          parsed_args.source_password)

        registry_url_arg = parsed_args.registry_url
        if registry_url_arg is None:
            registry_url_arg = image_uploader.get_undercloud_registry()
        if not registry_url_arg.startswith('docker://'):
            registry_url = 'docker://%s' % registry_url_arg
        else:
            registry_url = registry_url_arg
        reg_url = parse.urlparse(registry_url)

        session = uploader.authenticate(reg_url,
                                        parsed_args.username,
                                        parsed_args.password)
        try:
            if not parsed_args.dry_run:
                task = image_uploader.UploadTask(
                    image_name=image_name,
                    pull_source=image_source,
                    push_destination=registry_url_arg,
                    append_tag=parsed_args.append_tag,
                    modify_role=None,
                    modify_vars=None,
                    cleanup=parsed_args.cleanup,
                    multi_arch=parsed_args.multi_arch)

                uploader.add_upload_task(task)
                uploader.run_tasks()
        except OSError as e:
            if e.errno == errno.EACCES:
                self.log.error("Unable to upload due to permissions. "
                               "Please prefix command with sudo.")
            raise oscexc.CommandError(e)
        finally:
            session.close()


class TripleOContainerImageDelete(command.Command):
    """Delete specified image from registry."""

    auth_required = False
    log = logging.getLogger(__name__ + ".TripleoContainerImageDelete")

    def get_parser(self, prog_name):
        parser = super(TripleOContainerImageDelete, self).get_parser(prog_name)
        parser.add_argument(
            "--registry-url",
            dest="registry_url",
            metavar='<registry url>',
            default=None,
            help=_("URL of registry images are to be listed from in the "
                   "form <fqdn>:<port>.")
        )
        parser.add_argument(
            dest="image_to_delete",
            metavar='<image to delete>',
            help=_("Full URL of image to be deleted in the "
                   "form <fqdn>:<port>/path/to/image")
        )
        parser.add_argument(
            "--username",
            dest="username",
            metavar='<username>',
            help=_("Username for image registry.")
        )
        parser.add_argument(
            "--password",
            dest="password",
            metavar='<password>',
            help=_("Password for image registry.")
        )
        parser.add_argument(
            '-y', '--yes',
            help=_('Skip yes/no prompt (assume yes).'),
            default=False,
            action="store_true")
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)

        if not parsed_args.yes:
            confirm = utils.prompt_user_for_confirmation(
                    message=_("Are you sure you want to delete this image "
                              "[y/N]? "),
                    logger=self.log)
            if not confirm:
                raise oscexc.CommandError("Action not confirmed, exiting.")

        lock = processlock.ProcessLock()
        manager = image_uploader.ImageUploadManager(lock=lock)
        uploader = manager.uploader('python')
        registry_url_arg = parsed_args.registry_url
        if registry_url_arg is None:
            registry_url_arg = image_uploader.get_undercloud_registry()
        url = uploader._image_to_url(registry_url_arg)
        session = uploader.authenticate(url, parsed_args.username,
                                        parsed_args.password)

        try:
            uploader.delete(parsed_args.image_to_delete, session=session)
        except OSError as e:
            if e.errno == errno.EACCES:
                self.log.error("Unable to remove due to permissions. "
                               "Please prefix command with sudo.")
            raise oscexc.CommandError(e)
        finally:
            session.close()


class TripleOContainerImageList(command.Lister):
    """List images discovered in registry."""

    auth_required = False
    log = logging.getLogger(__name__ + ".TripleoContainerImageList")

    def get_parser(self, prog_name):
        parser = super(TripleOContainerImageList, self).get_parser(prog_name)
        parser.add_argument(
            "--registry-url",
            dest="registry_url",
            metavar='<registry url>',
            default=None,
            help=_("URL of registry images are to be listed from in the "
                   "form <fqdn>:<port>.")
        )
        parser.add_argument(
            "--username",
            dest="username",
            metavar='<username>',
            help=_("Username for image registry.")
        )
        parser.add_argument(
            "--password",
            dest="password",
            metavar='<password>',
            help=_("Password for image registry.")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)

        lock = processlock.ProcessLock()
        manager = image_uploader.ImageUploadManager(lock=lock)
        uploader = manager.uploader('python')
        registry_url_arg = parsed_args.registry_url
        if registry_url_arg is None:
            registry_url_arg = image_uploader.get_undercloud_registry()
        url = uploader._image_to_url(registry_url_arg)
        session = uploader.authenticate(url, parsed_args.username,
                                        parsed_args.password)
        try:
            results = uploader.list(url.geturl(), session=session)
        finally:
            session.close()

        cliff_results = []
        for r in results:
            cliff_results.append((r,))
        return (("Image Name",), cliff_results)


class TripleOContainerImageShow(command.ShowOne):
    """Show image selected from the registry."""

    auth_required = False
    log = logging.getLogger(__name__ + ".TripleoContainerImageShow")

    @property
    def formatter_default(self):
        return 'json'

    def get_parser(self, prog_name):
        parser = super(TripleOContainerImageShow, self).get_parser(prog_name)
        parser.add_argument(
            "--username",
            dest="username",
            metavar='<username>',
            help=_("Username for image registry.")
        )
        parser.add_argument(
            "--password",
            dest="password",
            metavar='<password>',
            help=_("Password for image registry.")
        )
        parser.add_argument(
            dest="image_to_inspect",
            metavar='<image to inspect>',
            help=_(
                "Image to be inspected, for example: "
                "docker.io/library/centos:7 or "
                "docker://docker.io/library/centos:7")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)

        lock = processlock.ProcessLock()
        manager = image_uploader.ImageUploadManager(lock=lock)
        uploader = manager.uploader('python')
        url = uploader._image_to_url(parsed_args.image_to_inspect)
        session = uploader.authenticate(url, parsed_args.username,
                                        parsed_args.password)
        try:
            image_inspect_result = uploader.inspect(
                parsed_args.image_to_inspect,
                session=session)
        finally:
            session.close()

        return self.format_image_inspect(image_inspect_result)

    def format_image_inspect(self, image_inspect_result):
        column_names = ['Name']
        data = [image_inspect_result.pop('Name')]

        result_fields = list(image_inspect_result.keys())
        result_fields.sort()
        for field in result_fields:
            column_names.append(field)
            data.append(image_inspect_result[field])

        return column_names, data


class TripleOImagePrepareDefault(command.Command):
    """Generate a default ContainerImagePrepare parameter."""

    auth_required = False
    log = logging.getLogger(__name__ + ".TripleoImagePrepare")

    def get_parser(self, prog_name):
        parser = super(TripleOImagePrepareDefault, self).get_parser(prog_name)
        parser.add_argument(
            "--output-env-file",
            dest="output_env_file",
            metavar='<file path>',
            help=_("File to write environment file containing default "
                   "ContainerImagePrepare value."),
        )
        parser.add_argument(
            '--local-push-destination',
            dest='push_destination',
            action='store_true',
            default=False,
            help=_('Include a push_destination to trigger upload to a local '
                   'registry.')
        )
        parser.add_argument(
            '--enable-registry-login',
            dest='registry_login',
            action='store_true',
            default=False,
            help=_('Use this flag to enable the flag to have systems attempt '
                   'to login to a remote registry prior to pulling their '
                   'containers. This flag should be used when '
                   '--local-push-destination is *NOT* used and the target '
                   'systems will have network connectivity to the remote '
                   'registries. Do not use this for an overcloud that '
                   'may not have network connectivity to a remote registry.')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)

        cip = copy.deepcopy(kolla_builder.CONTAINER_IMAGE_PREPARE_PARAM)
        if parsed_args.push_destination:
            for entry in cip:
                entry['push_destination'] = True
        params = {
            'ContainerImagePrepare': cip
        }
        if parsed_args.registry_login:
            if parsed_args.push_destination:
                self.log.warning('[WARNING] --local-push-destination was used '
                                 'with --enable-registry-login. Please make '
                                 'sure you understand the use of these '
                                 'parameters together as they can cause '
                                 'deployment failures.')
            self.log.warning('[NOTE] Make sure to update the paramter_defaults'
                             ' with ContainerImageRegistryCredentials for the '
                             'registries requiring authentication.')
            params['ContainerImageRegistryLogin'] = True

        env_data = build_env_file(params, self.app.command_options)
        self.app.stdout.write(env_data)
        if parsed_args.output_env_file:
            if os.path.exists(parsed_args.output_env_file):
                self.log.warning("Output env file exists, "
                                 "moving it to backup.")
                shutil.move(parsed_args.output_env_file,
                            parsed_args.output_env_file + ".backup")
            utils.safe_write(parsed_args.output_env_file, env_data)


class TripleOImagePrepare(command.Command):
    """Prepare and upload containers from a single command."""

    auth_required = False
    log = logging.getLogger(__name__ + ".TripleoImagePrepare")

    def get_parser(self, prog_name):
        parser = super(TripleOImagePrepare, self).get_parser(prog_name)
        try:
            roles_file = utils.rel_or_abs_path(
                constants.OVERCLOUD_ROLES_FILE,
                constants.TRIPLEO_HEAT_TEMPLATES)
        except exceptions.DeploymentError:
            roles_file = None
        parser.add_argument(
            '--environment-file', '-e', metavar='<file path>',
            action='append', dest='environment_files',
            help=_('Environment file containing the ContainerImagePrepare '
                   'parameter which specifies all prepare actions. '
                   'Also, environment files specifying which services are '
                   'containerized. Entries will be filtered to only contain '
                   'images used by containerized services. (Can be specified '
                   'more than once.)')
        )
        parser.add_argument(
            '--environment-directory', metavar='<HEAT ENVIRONMENT DIRECTORY>',
            action='append', dest='environment_directories',
            default=[os.path.expanduser(constants.DEFAULT_ENV_DIRECTORY)],
            help=_('Environment file directories that are automatically '
                   'added to the environment. '
                   'Can be specified more than once. Files in directories are '
                   'loaded in ascending sort order.')
        )
        parser.add_argument(
            '--roles-file', '-r', dest='roles_file',
            default=roles_file,
            help=_(
                'Roles file, overrides the default %s in the t-h-t templates '
                'directory used for deployment. May be an '
                'absolute path or the path relative to the templates dir.'
                ) % constants.OVERCLOUD_ROLES_FILE
        )
        parser.add_argument(
            "--output-env-file",
            dest="output_env_file",
            metavar='<file path>',
            help=_("File to write heat environment file which specifies all "
                   "image parameters. Any existing file will be overwritten."),
        )
        parser.add_argument(
            '--dry-run',
            dest='dry_run',
            action='store_true',
            default=False,
            help=_('Do not perform any pull, modify, or push operations. '
                   'The environment file will still be populated as if these '
                   'operations were performed.')
        )
        parser.add_argument(
            "--cleanup",
            dest="cleanup",
            metavar='<full, partial, none>',
            default=image_uploader.CLEANUP_FULL,
            help=_("Cleanup behavior for local images left after upload. "
                   "The default 'full' will attempt to delete all local "
                   "images. 'partial' will leave images required for "
                   "deployment on this host. 'none' will do no cleanup.")
        )
        parser.add_argument(
            "--log-file",
            dest="log_file",
            default=constants.CONTAINER_IMAGE_PREPARE_LOG_FILE,
            help=_("Log file to be used for python logging. "
                   "By default it would be logged to "
                   "$HOME/container_image_prepare.log.")
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)" % parsed_args)

        if parsed_args.cleanup not in image_uploader.CLEANUP:
            raise oscexc.CommandError('--cleanup must be one of: %s' %
                                      ', '.join(image_uploader.CLEANUP))

        role_file = None
        if parsed_args.roles_file:
            role_file = utils.rel_or_abs_path(parsed_args.roles_file,
                                              constants.TRIPLEO_HEAT_TEMPLATES)
        env_dirs = [os.path.abspath(x)
                    for x in parsed_args.environment_directories]
        env_files = [os.path.abspath(x)
                     for x in parsed_args.environment_files]
        extra_vars = {
            "roles_file": role_file,
            "environment_directories": env_dirs,
            "environment_files": env_files,
            "cleanup": parsed_args.cleanup,
            "dry_run": parsed_args.dry_run,
            "log_file": parsed_args.log_file}

        if self.app_args.verbose_level >= 3:
            extra_vars["debug"] = True

        if parsed_args.output_env_file:
            extra_vars["output_env_file"] = parsed_args.output_env_file

        with oooutils.TempDirs() as tmp:
            oooutils.run_ansible_playbook(
                playbook='cli-container-image-prepare.yaml',
                inventory='localhost,',
                workdir=tmp,
                playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
                verbosity=oooutils.playbook_verbosity(self=self),
                extra_vars=extra_vars)



#   Copyright 2019 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Plugin action implementation"""

import argparse
import logging
import subprocess

from openstackclient.i18n import _

from oslo_config import cfg

from tripleoclient import command
from tripleoclient import constants
from tripleoclient import exceptions
from tripleoclient import utils
from tripleoclient.v1 import minion_config

MINION_FAILURE_MESSAGE = """
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

An error has occured while deploying the Undercloud Minion

See the previous output for details about what went wrong.

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"""

MINION_COMPLETION_MESSAGE = """
##########################################################

The Undercloud Minion has been successfully installed.

##########################################################
"""
MINION_UPGRADE_COMPLETION_MESSAGE = """
##########################################################

The Undercloud Minion has been successfully upgraded.

##########################################################
"""


class InstallUndercloudMinion(command.Command):
    """Install and setup the undercloud minion"""

    auth_required = False
    log = logging.getLogger(__name__ + ".InstallUndercloudMinion")
    osloconfig = cfg.CONF

    def get_parser(self, prog_name):
        parser = argparse.ArgumentParser(
            description=self.get_description(),
            prog=prog_name,
            add_help=False
        )
        parser.add_argument('--force-stack-update',
                            dest='force_stack_update',
                            action='store_true',
                            default=False,
                            help=_("Do a virtual update of the ephemeral "
                                   "heat stack. New or failed deployments "
                                   "always have the stack_action=CREATE. This "
                                   "option enforces stack_action=UPDATE."),
                            )
        parser.add_argument(
            '--no-validations',
            dest='no_validations',
            action='store_true',
            default=False,
            help=_("Do not perform minion configuration validations"),
        )
        parser.add_argument(
            '--dry-run',
            dest='dry_run',
            action='store_true',
            default=False,
            help=_("Print the install command instead of running it"),
        )
        parser.add_argument('-y', '--yes', default=False,
                            action='store_true',
                            help=_("Skip yes/no prompt (assume yes)."))
        return parser

    def take_action(self, parsed_args):
        # Fetch configuration used to add logging to a file
        utils.load_config(self.osloconfig, constants.MINION_CONF_PATH)
        utils.configure_logging(self.log, self.app_args.verbose_level,
                                self.osloconfig['minion_log_file'])
        self.log.debug("take_action(%s)" % parsed_args)

        utils.ensure_run_as_normal_user()
        no_validations = parsed_args.dry_run or parsed_args.no_validations
        cmd = minion_config.prepare_minion_deploy(
            no_validations=no_validations,
            verbose_level=self.app_args.verbose_level,
            force_stack_update=parsed_args.force_stack_update,
            dry_run=parsed_args.dry_run)

        self.log.warning("Running: %s" % ' '.join(cmd))
        if not parsed_args.dry_run:
            try:
                subprocess.check_call(cmd)
                self.log.warning(MINION_COMPLETION_MESSAGE)
            except Exception as e:
                self.log.error(MINION_FAILURE_MESSAGE)
                self.log.error(e)
                raise exceptions.DeploymentError(e)


class UpgradeUndercloudMinion(InstallUndercloudMinion):
    """Upgrade undercloud minion"""

    auth_required = False
    log = logging.getLogger(__name__ + ".UpgradeUndercloudMinion")
    osloconfig = cfg.CONF

    def take_action(self, parsed_args):
        # Fetch configuration used to add logging to a file
        utils.load_config(self.osloconfig, constants.MINION_CONF_PATH)
        utils.configure_logging(self.log, self.app_args.verbose_level,
                                self.osloconfig['minion_log_file'])
        self.log.debug("take action(%s)" % parsed_args)

        utils.ensure_run_as_normal_user()
        cmd = minion_config.\
            prepare_minion_deploy(
                upgrade=True,
                yes=parsed_args.yes,
                no_validations=parsed_args.
                no_validations,
                verbose_level=self.app_args.verbose_level,
                force_stack_update=parsed_args.force_stack_update)
        self.log.warning("Running: %s" % ' '.join(cmd))
        if not parsed_args.dry_run:
            try:
                subprocess.check_call(cmd)
                self.log.warning(MINION_UPGRADE_COMPLETION_MESSAGE)
            except Exception as e:
                self.log.error(MINION_FAILURE_MESSAGE)
                self.log.error(e)
                raise exceptions.DeploymentError(e)



#   Copyright 2020 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import os

import mock

from osc_lib.tests import utils

from tripleoclient.v1 import overcloud_export_ceph


class TestOvercloudExportCeph(utils.TestCommand):

    def setUp(self):
        super(TestOvercloudExportCeph, self).setUp()

        self.cmd = overcloud_export_ceph.ExportOvercloudCeph(self.app, None)
        self.tripleoclient = mock.Mock()
        self.app.client_manager.tripleoclient = self.tripleoclient
        self.mock_open = mock.mock_open()

    @mock.patch('os.path.exists')
    @mock.patch('yaml.safe_dump')
    @mock.patch('tripleoclient.export.export_ceph')
    def test_export_ceph(self, mock_export_ceph,
                         mock_safe_dump,
                         mock_exists):
        argslist = ['--stack', 'dcn0']
        verifylist = [('stack', 'dcn0')]
        parsed_args = self.check_parser(self.cmd, argslist, verifylist)
        mock_exists.return_value = False
        expected = {
            'external_cluster_mon_ips': '192.168.24.42',
            'keys': [
                {'name': 'client.openstack'}
            ],
            'ceph_conf_overrides': {
                'client': {
                    'keyring': '/etc/ceph/dcn0.client.openstack.keyring'
                }
            },
            'cluster': 'dcn0',
            'fsid': 'a5a22d37-e01f-4fa0-a440-c72585c7487f',
            'dashboard_enabled': False
        }
        data = {}
        data['parameter_defaults'] = {}
        data['parameter_defaults']['CephExternalMultiConfig'] = [expected]
        mock_export_ceph.return_value = expected

        with mock.patch('six.moves.builtins.open', self.mock_open):
            self.cmd.take_action(parsed_args)
        path = os.path.join(os.environ.get('HOME'), 'config-download')
        mock_export_ceph.assert_called_once_with('dcn0', 'openstack', path)
        self.assertEqual(data, mock_safe_dump.call_args[0][0])



#   Copyright 2018 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from tripleoclient.config.standalone import StandaloneConfig
from tripleoclient.tests import base


class TestStandaloneConfig(base.TestCase):
    def setUp(self):
        super(TestStandaloneConfig, self).setUp()
        # Get the class object to test
        self.config = StandaloneConfig()

    def test_get_base_opts(self):
        ret = self.config.get_base_opts()
        expected = ['cleanup',
                    'container_cli',
                    'container_healthcheck_disabled',
                    'container_images_file',
                    'container_insecure_registries',
                    'container_registry_mirror',
                    'custom_env_files',
                    'deployment_user',
                    'heat_container_image',
                    'heat_native',
                    'hieradata_override',
                    'net_config_override',
                    'networks_file',
                    'output_dir',
                    'roles_file',
                    'templates']
        self.assertEqual(expected, [x.name for x in ret])

    def test_get_service_opts(self):
        ret = self.config.get_enable_service_opts()
        expected = ['enable_cinder',
                    'enable_ironic',
                    'enable_ironic_inspector',
                    'enable_mistral',
                    'enable_nova',
                    'enable_novajoin',
                    'enable_swift',
                    'enable_telemetry',
                    'enable_tempest',
                    'enable_validations',
                    'enable_zaqar']
        self.assertEqual(expected, [x.name for x in ret])
        for x in ret:
            self.assertEqual(x.default, False, "%s config not False" % x.name)

    def test_get_service_opts_enabled(self):
        ret = self.config.get_enable_service_opts(cinder=True,
                                                  ironic=True,
                                                  ironic_inspector=True,
                                                  mistral=True,
                                                  nova=True,
                                                  novajoin=True,
                                                  swift=True,
                                                  telemetry=True,
                                                  tempest=True,
                                                  validations=True,
                                                  zaqar=True)
        expected = ['enable_cinder',
                    'enable_ironic',
                    'enable_ironic_inspector',
                    'enable_mistral',
                    'enable_nova',
                    'enable_novajoin',
                    'enable_swift',
                    'enable_telemetry',
                    'enable_tempest',
                    'enable_validations',
                    'enable_zaqar']
        self.assertEqual(expected, [x.name for x in ret])
        for x in ret:
            self.assertEqual(x.default, True, "%s config not True" % x.name)

    def test_get_opts(self):
        ret = self.config.get_opts()
        expected = ['cleanup',
                    'container_cli',
                    'container_healthcheck_disabled',
                    'container_images_file',
                    'container_insecure_registries',
                    'container_registry_mirror',
                    'custom_env_files',
                    'deployment_user',
                    'enable_cinder',
                    'enable_ironic',
                    'enable_ironic_inspector',
                    'enable_mistral',
                    'enable_nova',
                    'enable_novajoin',
                    'enable_swift',
                    'enable_telemetry',
                    'enable_tempest',
                    'enable_validations',
                    'enable_zaqar',
                    'heat_container_image',
                    'heat_native',
                    'hieradata_override',
                    'net_config_override',
                    'networks_file',
                    'output_dir',
                    'roles_file',
                    'templates']
        self.assertEqual(expected, [x.name for x in ret])



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import mock

from osc_lib.tests import utils

from tripleoclient.tests import fakes
from tripleoclient.v1 import overcloud_config


class TestOvercloudConfig(utils.TestCommand):

    def setUp(self):
        super(TestOvercloudConfig, self).setUp()
        self.cmd = overcloud_config.DownloadConfig(self.app, None)
        self.app.client_manager.orchestration = mock.Mock()
        self.app.options = fakes.FakeOptions()

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    def test_overcloud_download_config(self, mock_playbook):
        arglist = ['--name', 'overcloud', '--config-dir', '/tmp']
        verifylist = [
            ('name', 'overcloud'),
            ('config_dir', '/tmp'),
            ('preserve_config_dir', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        mock_playbook.assert_called_once_with(
            extra_vars={'plan': 'overcloud', 'config_dir': '/tmp',
                        'preserve_config': True},
            inventory='localhost,', playbook='cli-config-download-export.yaml',
            playbook_dir='/usr/share/ansible/tripleo-playbooks',
            verbosity=3, workdir=mock.ANY)

    @mock.patch("tripleoclient.utils.run_ansible_playbook", autospec=True)
    def test_overcloud_download_config_no_preserve(self, mock_playbook):
        arglist = ['--name', 'overcloud', '--config-dir', '/tmp',
                   '--no-preserve-config']
        verifylist = [
            ('name', 'overcloud'),
            ('config_dir', '/tmp'),
            ('preserve_config_dir', False)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        mock_playbook.assert_called_once_with(
            extra_vars={'plan': 'overcloud', 'config_dir': '/tmp',
                        'preserve_config': False},
            inventory='localhost,', playbook='cli-config-download-export.yaml',
            playbook_dir='/usr/share/ansible/tripleo-playbooks',
            verbosity=3, workdir=mock.ANY)



#   Copyright 2017 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import logging

from osc_lib.i18n import _

from tripleoclient import command
from tripleoclient import constants
from tripleoclient import utils


class ReportExecute(command.Command):
    """Run sosreport on selected servers."""

    log = logging.getLogger(__name__ + ".ReportExecute")

    def get_parser(self, prog_name):
        parser = super(ReportExecute, self).get_parser(prog_name)
        parser.add_argument('server_name',
                            help=_('Server name, group name, or partial name'
                                   ' to match. For example "Controller" will'
                                   ' match all controllers for an'
                                   ' environment.'))
        parser.add_argument('--stack',
                            help=_("Stack name to use for log collection."),
                            default='overcloud')
        # Deprecated in U
        parser.add_argument('-c',
                            '--container',
                            dest='container',
                            default=None,
                            help=_('This option no-longer has any effect.'))
        parser.add_argument('-o',
                            '--output',
                            dest='destination',
                            default='/var/lib/tripleo/support',
                            help=_('Output directory for the report'))
        # Deprecated in U
        parser.add_argument('--skip-container-delete',
                            dest='skip_delete',
                            default=False,
                            help=_('This option no-longer has any effect.'),
                            action='store_true')
        # Deprecated in U
        parser.add_argument('-t',
                            '--timeout',
                            dest='timeout',
                            type=int,
                            default=None,
                            help=_('This option no-longer has any effect.'))
        # Deprecated in U
        parser.add_argument('-n',
                            '--concurrency',
                            dest='concurrency',
                            type=int,
                            default=None,
                            help=_('This option no-longer has any effect.'))
        # Deprecated in U
        parser.add_argument('--collect-only',
                            dest='collect_only',
                            help=_('This option no-longer has any effect.'),
                            default=False,
                            action='store_true')
        # Deprecated in U
        parser.add_argument('--download-only',
                            dest='download_only',
                            help=_('This option no-longer has any effect.'),
                            default=False,
                            action='store_true')
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action({})'.format(parsed_args))

        extra_vars = {
            'server_name': parsed_args.server_name,
            'sos_destination': parsed_args.destination,
        }

        with utils.TempDirs() as tmp:
            utils.run_ansible_playbook(
                playbook='cli-support-collect-logs.yaml',
                inventory=constants.ANSIBLE_INVENTORY.format(
                    parsed_args.stack
                ),
                workdir=tmp,
                playbook_dir=constants.ANSIBLE_TRIPLEO_PLAYBOOKS,
                verbosity=utils.playbook_verbosity(self=self),
                extra_vars=extra_vars
            )



#   Copyright 2015 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from datetime import datetime
import mock
import os

from osc_lib import exceptions
import tripleo_common.arch
from tripleoclient.tests import base
from tripleoclient.tests.fakes import FakeHandle
from tripleoclient.tests.v1.test_plugin import TestPluginV1
from tripleoclient import utils as plugin_utils
from tripleoclient.v1 import overcloud_image


class TestOvercloudImageBuild(TestPluginV1):

    def setUp(self):
        super(TestOvercloudImageBuild, self).setUp()

        run_cmd = mock.patch('tripleoclient.utils.run_command')
        self.mock_run_command = run_cmd.start()
        self.addCleanup(run_cmd.stop)
        # Get the command object to test
        self.cmd = overcloud_image.BuildOvercloudImage(self.app, None)

    @mock.patch('tripleo_common.image.build.ImageBuildManager', autospec=True)
    def test_overcloud_image_build_default_yaml(self, mock_manager):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        mock_manager.assert_called_once_with(
            ['/usr/share/openstack-tripleo-common/image-yaml/'
             'overcloud-images-python3.yaml',
             '/usr/share/openstack-tripleo-common/image-yaml/'
             'overcloud-images-centos8.yaml'],
            output_directory='.',
            skip=True,
            images=None)
        cmd = ['sudo', 'dnf', 'install', '-y'] + self.cmd.REQUIRED_PACKAGES
        self.mock_run_command.assert_called_once_with(
            cmd, name="Install required packages")

    @mock.patch('tripleo_common.image.build.ImageBuildManager', autospec=True)
    def test_overcloud_image_build_yaml(self, mock_manager):
        arglist = ['--config-file', 'config.yaml']
        verifylist = [('config_files', ['config.yaml'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        mock_manager.assert_called_once_with(
            ['config.yaml'],
            output_directory='.',
            skip=True,
            images=None)

    @mock.patch('tripleo_common.image.build.ImageBuildManager', autospec=True)
    def test_overcloud_image_build_multi_yaml(self, mock_manager):
        arglist = ['--config-file', 'config1.yaml',
                   '--config-file', 'config2.yaml']
        verifylist = [('config_files', ['config1.yaml', 'config2.yaml'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        mock_manager.assert_called_once_with(
            ['config1.yaml', 'config2.yaml'],
            output_directory='.',
            skip=True,
            images=None)

    @mock.patch('tripleo_common.image.build.ImageBuildManager', autospec=True)
    def test_overcloud_image_build_with_no_skip(self, mock_manager):
        arglist = ['--config-file', 'config.yaml', '--no-skip']
        verifylist = [('config_files', ['config.yaml']),
                      ('skip', False)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        mock_manager.assert_called_once_with(
            ['config.yaml'],
            output_directory='.',
            skip=False,
            images=None)

    @mock.patch('tripleo_common.image.build.ImageBuildManager', autospec=True)
    def test_overcloud_image_build_with_output_directory(self, mock_manager):
        arglist = ['--config-file', 'config.yaml',
                   '--output-directory', '/tmp/abc']
        verifylist = [('config_files', ['config.yaml']),
                      ('output_directory', '/tmp/abc')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        mock_manager.assert_called_once_with(
            ['config.yaml'],
            output_directory='/tmp/abc',
            skip=True,
            images=None)


class TestBaseClientAdapter(base.TestCommand):

    def setUp(self):
        super(TestBaseClientAdapter, self).setUp()
        self.adapter = overcloud_image.BaseClientAdapter('/foo')

    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._copy_file', autospec=True)
    def test_file_try_update_need_update(self,
                                         mock_copy_file,
                                         mock_files_changed,
                                         mock_isfile):
        mock_isfile.return_value = True
        mock_files_changed.return_value = True

        self.adapter.file_create_or_update('discimg', 'discimgprod')
        mock_copy_file.assert_not_called()

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._copy_file', autospec=True)
    def test_file_try_update_do_update(self,
                                       mock_copy_file,
                                       mock_files_changed):
        mock_files_changed.return_value = True

        self.update_existing = True
        self.adapter.file_create_or_update('discimg', 'discimgprod')
        mock_copy_file.assert_called_once_with(
            self.adapter, 'discimg', 'discimgprod')

    @mock.patch('subprocess.check_call', autospec=True)
    def test_copy_file(self, mock_subprocess_call):
        self.adapter._copy_file('/foo.qcow2', 'bar.qcow2')
        mock_subprocess_call.assert_called_once_with(
            'sudo cp -f "/foo.qcow2" "bar.qcow2"', shell=True)

    @mock.patch('subprocess.check_call', autospec=True)
    def test_move_file(self, mock_subprocess_call):
        self.adapter._move_file('/foo.qcow2', 'bar.qcow2')
        mock_subprocess_call.assert_called_once_with(
            'sudo mv "/foo.qcow2" "bar.qcow2"', shell=True)

    @mock.patch('subprocess.check_call', autospec=True)
    def test_make_dirs(self, mock_subprocess_call):
        self.adapter._make_dirs('/foo/bar/baz')
        mock_subprocess_call.assert_called_once_with(
            'sudo mkdir -m 0775 -p "/foo/bar/baz"', shell=True)


class TestFileImageClientAdapter(TestPluginV1):

    def setUp(self):
        super(TestFileImageClientAdapter, self).setUp()
        self.updated = []
        self.adapter = overcloud_image.FileImageClientAdapter(
            image_path='/home/foo',
            local_path='/my/images',
            updated=self.updated
        )
        self.image = mock.Mock()
        self.image.id = 'file:///my/images/x86_64/overcloud-full.qcow2'
        self.image.name = 'overcloud-full'
        self.image.checksum = 'asdf'
        self.image.created_at = '2019-11-14T01:33:39'
        self.image.size = 982802432

    @mock.patch('os.path.exists')
    def test_get_image_property(self, mock_exists):
        mock_exists.side_effect = [
            True, True, False, False
        ]
        image = mock.Mock()
        image.id = 'file:///my/images/x86_64/overcloud-full.qcow2'
        # file exists
        self.assertEqual(
            'file:///my/images/x86_64/overcloud-full.vmlinuz',
            self.adapter.get_image_property(image, 'kernel_id')
        )
        self.assertEqual(
            'file:///my/images/x86_64/overcloud-full.initrd',
            self.adapter.get_image_property(image, 'ramdisk_id')
        )
        # file doesn't exist
        self.assertIsNone(
            self.adapter.get_image_property(image, 'kernel_id')
        )
        self.assertIsNone(
            self.adapter.get_image_property(image, 'ramdisk_id')
        )
        self.assertRaises(ValueError, self.adapter.get_image_property,
                          image, 'foo')

    def test_paths(self):
        self.assertEqual(
            ('/my/images/x86_64',
             'overcloud-full.vmlinuz'),
            self.adapter._paths(
                'overcloud-full',
                plugin_utils.overcloud_kernel,
                'x86_64',
                None
            )
        )
        self.assertEqual(
            ('/my/images',
             'overcloud-full.raw'),
            self.adapter._paths(
                'overcloud-full',
                plugin_utils.overcloud_image,
                None,
                None
            )
        )
        self.assertEqual(
            ('/my/images/power9-ppc64le',
             'overcloud-full.initrd'),
            self.adapter._paths(
                'overcloud-full',
                plugin_utils.overcloud_ramdisk,
                'ppc64le',
                'power9'
            )
        )

    @mock.patch('os.path.exists')
    @mock.patch('os.stat')
    @mock.patch('tripleoclient.utils.file_checksum')
    def test_get_image(self, mock_checksum, mock_stat, mock_exists):
        st_mtime = 1573695219
        mock_exists.return_value = True
        mock_stat.return_value.st_size = 982802432
        mock_stat.return_value.st_mtime = st_mtime
        mock_checksum.return_value = 'asdf'

        image = self.adapter._get_image(
            '/my/images/x86_64/overcloud-full.qcow2')
        self.assertEqual(
            'file:///my/images/x86_64/overcloud-full.qcow2',
            image.id
        )
        self.assertEqual('overcloud-full', image.name)
        self.assertEqual('asdf', image.checksum)
        self.assertEqual(datetime.fromtimestamp(st_mtime).strftime("%M:%S"),
                         datetime.strptime(
            image.created_at, "%Y-%m-%dT%H:%M:%S").strftime("%M:%S"))
        self.assertEqual(982802432, image.size)

    @mock.patch('tripleoclient.utils.file_checksum')
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'FileImageClientAdapter._get_image', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._move_file', autospec=True)
    def test_image_try_update(self, mock_move, mock_get_image, mock_checksum):

        # existing image with identical checksum
        mock_checksum.return_value = 'asdf'
        mock_get_image.return_value = self.image
        self.assertEqual(
            self.image,
            self.adapter._image_try_update(
                '/home/foo/overcloud-full.qcow2',
                '/my/images/x86_64/overcloud-full.qcow2'
            )
        )
        self.assertEqual([], self.updated)

        # no image to update
        mock_get_image.return_value = None
        self.assertIsNone(
            self.adapter._image_try_update(
                '/home/foo/overcloud-full.qcow2',
                '/my/images/x86_64/overcloud-full.qcow2'
            )
        )
        self.assertEqual([], self.updated)

        # existing image with different checksum, but update_existing=False
        mock_checksum.return_value = 'fdsa'
        mock_get_image.return_value = self.image
        self.assertEqual(
            self.image,
            self.adapter._image_try_update(
                '/home/foo/overcloud-full.qcow2',
                '/my/images/x86_64/overcloud-full.qcow2'
            )
        )
        self.assertEqual([], self.updated)

        # existing image with different checksum, update_existing=True
        self.adapter.update_existing = True
        self.assertIsNone(
            self.adapter._image_try_update(
                '/home/foo/overcloud-full.qcow2',
                '/my/images/x86_64/overcloud-full.qcow2'
            )
        )
        self.assertEqual(
            ['/my/images/x86_64/overcloud-full.qcow2'],
            self.updated
        )
        mock_move.assert_called_once_with(
            self.adapter,
            '/my/images/x86_64/overcloud-full.qcow2',
            '/my/images/x86_64/overcloud-full_20191114T013339.qcow2'
        )

    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'FileImageClientAdapter._get_image', autospec=True)
    @mock.patch('os.path.isdir')
    def test_upload_image(self, mock_isdir, mock_get_image,
                          mock_subprocess_call):
        mock_isdir.return_value = False

        mock_get_image.return_value = self.image

        result = self.adapter._upload_image(
            '/home/foo/overcloud-full.qcow2',
            '/my/images/x86_64/overcloud-full.qcow2'
        )
        self.assertEqual(self.image, result)
        mock_subprocess_call.assert_has_calls([
            mock.call('sudo mkdir -m 0775 -p "/my/images/x86_64"', shell=True),
            mock.call('sudo cp -f "/home/foo/overcloud-full.qcow2" '
                      '"/my/images/x86_64/overcloud-full.qcow2"', shell=True)
        ])

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'FileImageClientAdapter._upload_image', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'FileImageClientAdapter._image_try_update', autospec=True)
    def test_update_or_upload(self, mock_image_try_update, mock_upload_image):

        # image exists
        mock_image_try_update.return_value = self.image
        self.assertEqual(
            self.image,
            self.adapter.update_or_upload(
                image_name='overcloud-full',
                properties={},
                names_func=plugin_utils.overcloud_image,
                arch='x86_64'
            )
        )
        mock_upload_image.assert_not_called()

        # image needs uploading
        mock_image_try_update.return_value = None
        mock_upload_image.return_value = self.image
        self.assertEqual(
            self.image,
            self.adapter.update_or_upload(
                image_name='overcloud-full',
                properties={},
                names_func=plugin_utils.overcloud_image,
                arch='x86_64'
            )
        )
        mock_upload_image.assert_called_once_with(
            self.adapter,
            '/home/foo/overcloud-full.raw',
            '/my/images/x86_64/overcloud-full.raw'
        )
        mock_image_try_update.assert_has_calls([
            mock.call(self.adapter,
                      '/home/foo/overcloud-full.raw',
                      '/my/images/x86_64/overcloud-full.raw'),
            mock.call(self.adapter,
                      '/home/foo/overcloud-full.raw',
                      '/my/images/x86_64/overcloud-full.raw')
        ])


class TestGlanceClientAdapter(TestPluginV1):

    def setUp(self):
        super(TestGlanceClientAdapter, self).setUp()
        self.app.client_manager.image = mock.Mock()
        self.app.client_manager.image.version = 2.0
        self._arch = tripleo_common.arch.kernel_arch()
        self.app.client_manager.image.images.create.return_value = (
            mock.Mock(id=10, name='imgname',
                      properties={'kernel_id': 10, 'ramdisk_id': 10,
                                  'hw_architecture': self._arch},
                      created_at='2015-07-31T14:37:22.000000'))
        self.updated = []
        self.adapter = overcloud_image.GlanceClientAdapter(
            client=self.app.client_manager.image,
            image_path='/foo',
            updated=self.updated
        )

    def test_get_image_exists(self):
        image_mock = mock.Mock(name='imagename')
        self.app.client_manager.image.find_image.return_value = image_mock
        self.assertEqual(self.adapter._get_image('imagename'), image_mock)

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_image_try_update_no_exist(self, mock_get_image):
        mock_get_image.return_value = None
        self.assertFalse(self.adapter._image_try_update(
            'name', 'fn'))
        self.assertEqual([], self.updated)

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_image_try_update_need_update(self,
                                          mock_get_image,
                                          mock_image_changed):
        image_mock = mock.Mock(name='imagename')
        mock_get_image.return_value = image_mock
        mock_image_changed.return_value = True
        self.assertEqual(
            self.adapter._image_try_update('name', 'fn'),
            image_mock
        )
        self.assertEqual([], self.updated)
        self.app.client_manager.image.update_image.assert_not_called()

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_image_try_update_do_update(self,
                                        mock_get_image,
                                        mock_image_changed):
        image_mock = mock.Mock(name='imagename',
                               created_at='2015-07-31T14:37:22.000000')
        update_mock = mock.Mock(return_value=image_mock)
        self.app.client_manager.image.update_image = update_mock
        mock_get_image.return_value = image_mock
        mock_image_changed.return_value = True
        self.adapter.update_existing = True
        self.assertEqual(
            self.adapter._image_try_update('name', 'fn'),
            None
        )
        self.assertEqual([image_mock.id], self.updated)
        update_mock.assert_called_once()


class TestUploadOvercloudImage(TestPluginV1):

    def setUp(self):
        super(TestUploadOvercloudImage, self).setUp()

        # Get the command object to test
        self.cmd = overcloud_image.UploadOvercloudImage(self.app, None)
        self.app.client_manager.image = mock.Mock()
        self.app.client_manager.image.version = 2.0
        self._arch = tripleo_common.arch.kernel_arch()
        self.app.client_manager.image.create_image.return_value = (
            mock.Mock(id=10, name='imgname',
                      properties={'kernel_id': 10, 'ramdisk_id': 10,
                                  'hw_architecture': self._arch},
                      created_at='2015-07-31T14:37:22.000000'))
        mock_cfe = mock.patch('tripleoclient.v1.overcloud_image.'
                              'BaseClientAdapter.check_file_exists',
                              autospec=True)
        mock_cfe.start()
        self.addCleanup(mock_cfe.stop)
        mock_cfe.return_value = True

        mock_rifp = mock.patch('tripleoclient.v1.overcloud_image.'
                               'BaseClientAdapter.read_image_file_pointer',
                               autospec=True)
        mock_rifp.start()
        self.addCleanup(mock_rifp.stop)
        self._file_handle = FakeHandle()
        mock_rifp.return_value = self._file_handle

    @mock.patch.dict(os.environ, {'KEY': 'VALUE', 'OLD_KEY': 'OLD_VALUE'})
    def test_get_environment_var(self):
        self.assertEqual('default-value',
                         self.cmd._get_environment_var('MISSING',
                                                       'default-value'))
        self.assertEqual('VALUE',
                         self.cmd._get_environment_var('KEY',
                                                       'default-value'))
        self.assertEqual('VALUE',
                         self.cmd._get_environment_var('KEY',
                                                       'default-value',
                                                       deprecated=['MISSING']))
        self.assertEqual('OLD_VALUE',
                         self.cmd._get_environment_var('KEY',
                                                       'default-value',
                                                       deprecated=['OLD_KEY']))

    def test_platform_without_architecture_fail(self):
        parsed_args = self.check_parser(self.cmd, ['--platform', 'SNB'], [])
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_images_v2(self,
                                        mock_get_image,
                                        mock_subprocess_call,
                                        mock_isfile,
                                        mock_convert_image):
        parsed_args = self.check_parser(self.cmd, [], [])
        mock_isfile.return_value = False

        mock_get_image.return_value = None

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.images.delete.call_count
        )
        self.assertEqual(
            3,
            self.app.client_manager.image.create_image.call_count
        )
        self.app.client_manager.image.create_image.assert_has_calls([
            mock.call(name='overcloud-full-vmlinuz',
                      disk_format='aki',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
            mock.call(name='overcloud-full-initrd',
                      disk_format='ari',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
            mock.call(name='overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
        ])

        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 2)
        mock_subprocess_call.assert_has_calls([
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/var/lib/ironic/httpboot/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/var/lib/ironic/httpboot/agent.ramdisk"', shell=True)
        ])

    @mock.patch('os.path.isfile')
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_try_update', autospec=True)
    def test_overcloud_create_images_image_path(self,
                                                mock_image_try_update,
                                                mock_get_image,
                                                mock_subprocess_call,
                                                mock_isfile):
        parsed_args = self.check_parser(self.cmd,
                                        ['--image-path', '/foo'],
                                        [])
        mock_get_image.return_value = None
        mock_image_try_update.return_value = None
        mock_isfile.return_value = False

        self.cmd.take_action(parsed_args)

        self.cmd.adapter._image_try_update.assert_has_calls([
            mock.call(self.cmd.adapter,
                      'overcloud-full-vmlinuz',
                      '/foo/overcloud-full.vmlinuz'),
            mock.call(self.cmd.adapter,
                      'overcloud-full-initrd',
                      '/foo/overcloud-full.initrd'),
            mock.call(self.cmd.adapter,
                      'overcloud-full',
                      '/foo/overcloud-full.raw'),
        ])
        mock_subprocess_call.assert_has_calls([
            mock.call('sudo cp -f "/foo/ironic-python-agent.kernel" '
                      '"/var/lib/ironic/httpboot/agent.kernel"', shell=True),
            mock.call('sudo cp -f "/foo/ironic-python-agent.initramfs" '
                      '"/var/lib/ironic/httpboot/agent.ramdisk"', shell=True)
        ])

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    def test_overcloud_create_noupdate_images(self,
                                              mock_files_changed,
                                              mock_image_changed,
                                              mock_get_image,
                                              mock_subprocess_call,
                                              mock_isfile,
                                              mock_convert_image):
        parsed_args = self.check_parser(self.cmd, [], [])
        mock_isfile.return_value = True
        mock_files_changed.return_value = True

        existing_image = mock.Mock(id=10, name='imgname',
                                   properties={'kernel_id': 10,
                                               'ramdisk_id': 10})
        mock_get_image.return_value = existing_image
        mock_image_changed.return_value = True

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            0,
            self.app.client_manager.image.create_image.call_count
        )
        self.assertEqual(
            0,
            self.app.client_manager.image.image_update.call_count
        )

        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 0)
        self.assertFalse(self.cmd.updated)

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    def test_overcloud_create_update_images(self,
                                            mock_files_changed,
                                            mock_image_changed,
                                            mock_get_image,
                                            mock_subprocess_call,
                                            mock_convert_image):
        parsed_args = self.check_parser(self.cmd, ['--update-existing'], [])
        mock_files_changed.return_value = True

        existing_image = mock.Mock(id=10, name='imgname',
                                   properties={'kernel_id': 10,
                                               'ramdisk_id': 10},
                                   created_at='2015-07-31T14:37:22.000000')
        mock_get_image.return_value = existing_image
        mock_image_changed.return_value = True
        self.app.client_manager.image.image_update.return_value = (
            existing_image)

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            3,
            self.app.client_manager.image.create_image.call_count
        )
        self.assertEqual(
            6,  # 3 for new uploads, 3 updating the existsing
            self.app.client_manager.image.update_image.call_count
        )
        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 2)
        self.assertTrue(self.cmd.updated)


class TestUploadOvercloudImageFull(TestPluginV1):

    def setUp(self):
        super(TestUploadOvercloudImageFull, self).setUp()

        # Get the command object to test
        self.cmd = overcloud_image.UploadOvercloudImage(self.app, None)
        self.app.client_manager.image = mock.Mock()
        self.app.client_manager.image.version = 2.0
        self._arch = tripleo_common.arch.kernel_arch()
        self.app.client_manager.image.create_image.return_value = (
            mock.Mock(id=10, name='imgname',
                      properties={'hw_architecture': self._arch},
                      created_at='2015-07-31T14:37:22.000000'))
        mock_cfe = mock.patch('tripleoclient.v1.overcloud_image.'
                              'BaseClientAdapter.check_file_exists',
                              autospec=True)
        mock_cfe.start()
        self.addCleanup(mock_cfe.stop)
        mock_cfe.return_value = True

        mock_rifp = mock.patch('tripleoclient.v1.overcloud_image.'
                               'BaseClientAdapter.read_image_file_pointer',
                               autospec=True)
        mock_rifp.start()
        self.addCleanup(mock_rifp.stop)
        self._file_handle = FakeHandle()
        mock_rifp.return_value = self._file_handle

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_images(self,
                                     mock_get_image,
                                     mock_subprocess_call,
                                     mock_isfile, mock_convert_image):
        parsed_args = self.check_parser(self.cmd, ['--whole-disk'], [])
        mock_isfile.return_value = False

        mock_get_image.return_value = None

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            1,
            self.app.client_manager.image.create_image.call_count
        )

        self.app.client_manager.image.create_image.assert_has_calls([
            mock.call(name='overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public',),
        ])
        # properties are set by updating the image
        self.app.client_manager.image.update_image.assert_has_calls([
            mock.call(mock.ANY, hw_architecture=self._arch),
        ])

        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 2)
        mock_subprocess_call.assert_has_calls([
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/var/lib/ironic/httpboot/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/var/lib/ironic/httpboot/agent.ramdisk"', shell=True)
        ])

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_images_with_arch(self,
                                               mock_get_image,
                                               mock_subprocess_call,
                                               mock_isfile,
                                               mock_convert_image):
        parsed_args = self.check_parser(self.cmd,
                                        ['--whole-disk', '--arch', 'ppc64le'],
                                        [])
        mock_isfile.return_value = False

        mock_get_image.return_value = None

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            1,
            self.app.client_manager.image.create_image.call_count
        )

        self.app.client_manager.image.create_image.assert_has_calls([
            mock.call(name='ppc64le-overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
        ])

        self.app.client_manager.image.update_image.assert_has_calls([
            mock.call(mock.ANY, hw_architecture='ppc64le'),
        ])
        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 2)
        mock_subprocess_call.assert_has_calls([
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/var/lib/ironic/httpboot/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/var/lib/ironic/httpboot/agent.ramdisk"', shell=True)
        ])

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_noupdate_images(self, mock_get_image,
                                              mock_files_changed,
                                              mock_image_changed,
                                              mock_subprocess_call,
                                              mock_isfile,
                                              mock_convert_image):
        parsed_args = self.check_parser(self.cmd, ['--whole-disk'], [])
        mock_isfile.return_value = True
        mock_files_changed.return_value = True

        existing_image = mock.Mock(id=10, name='imgname',
                                   properties={'hw_architecture': self._arch})
        mock_get_image.return_value = existing_image
        self.cmd._image_changed = mock.Mock(return_value=True)
        mock_image_changed.return_value = True

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            0,
            self.app.client_manager.image.create_image.call_count
        )
        self.assertEqual(
            0,
            self.app.client_manager.image.update_image.call_count
        )

        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 0)

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_update_images(self, mock_get_image,
                                            mock_files_changed,
                                            mock_image_changed,
                                            mock_subprocess_call,
                                            mock_convert_image):
        parsed_args = self.check_parser(
            self.cmd, ['--update-existing', '--whole-disk'], [])
        mock_files_changed.return_value = True

        existing_image = mock.Mock(id=10, name='imgname',
                                   properties={'hw_architecture': self._arch},
                                   created_at='2015-07-31T14:37:22.000000')
        mock_get_image.return_value = existing_image
        mock_image_changed.return_value = True
        self.app.client_manager.image.update_image.return_value = (
            existing_image)

        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            1,
            self.app.client_manager.image.create_image.call_count
        )
        self.assertEqual(
            2,  # update 1 image *and* add properties to 1 image
            self.app.client_manager.image.update_image.call_count
        )
        self.assertEqual(mock_convert_image.call_count, 1)
        self.assertEqual(mock_subprocess_call.call_count, 2)


class TestUploadOvercloudImageFullMultiArch(TestPluginV1):
    # NOTE(tonyb): Really only the id is important below, but the names make
    # reading logfiles a little nicer
    images = [
        mock.Mock(id=10, name='overcloud-full'),
        mock.Mock(id=11, name='ppc64le-overcloud-full'),
        mock.Mock(id=12, name='p9-ppc64le-overcloud-full'),
    ]

    def setUp(self):
        super(TestUploadOvercloudImageFullMultiArch, self).setUp()

        # Get the command object to test
        self.cmd = overcloud_image.UploadOvercloudImage(self.app, None)
        self.app.client_manager.image = mock.Mock()
        self.app.client_manager.image.version = 2.0
        # NOTE(tonyb): This is a little fragile.  It works because
        # GlanceClientAdapter._upload_image() calls
        # self.client.images.create() and self.client.images.get() once each
        # call so this way we always create() and get() the same mocked "image"
        self.app.client_manager.image.create_image.side_effect = self.images
        self.app.client_manager.image.get_image.side_effect = self.images

        mock_cfe = mock.patch('tripleoclient.v1.overcloud_image.'
                              'BaseClientAdapter.check_file_exists',
                              autospec=True)
        mock_cfe.start()
        self.addCleanup(mock_cfe.stop)
        mock_cfe.return_value = True

        mock_rifp = mock.patch('tripleoclient.v1.overcloud_image.'
                               'BaseClientAdapter.read_image_file_pointer',
                               autospec=True)
        mock_rifp.start()
        self.addCleanup(mock_rifp.stop)
        self._file_handle = FakeHandle()
        mock_rifp.return_value = self._file_handle

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image', autospec=True)
    @mock.patch('tripleo_common.arch.kernel_arch', autospec=True,
                return_value='x86_64')
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_images_with_arch(self, mock_get_image,
                                               mock_subprocess_call,
                                               mock_isfile, mock_arch,
                                               mock_convert_image):
        mock_isfile.return_value = False
        mock_get_image.return_value = None

        parsed_args = self.check_parser(self.cmd,
                                        ['--whole-disk'],
                                        [])
        self.cmd.take_action(parsed_args)
        parsed_args = self.check_parser(self.cmd,
                                        ['--whole-disk',
                                         '--http-boot', '/httpboot/ppc64le',
                                         '--arch', 'ppc64le'],
                                        [])
        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            2,
            self.app.client_manager.image.create_image.call_count
        )

        self.app.client_manager.image.create_image.assert_has_calls([
            mock.call(name='overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
            mock.call(name='ppc64le-overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
        ])

        self.app.client_manager.image.update_image.assert_has_calls([
            mock.call(10, hw_architecture='x86_64'),
            mock.call(11, hw_architecture='ppc64le'),
        ])
        self.assertEqual(mock_convert_image.call_count, 2)
        self.assertEqual(mock_subprocess_call.call_count, 4)
        mock_subprocess_call.assert_has_calls([
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/var/lib/ironic/httpboot/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/var/lib/ironic/httpboot/agent.ramdisk"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/httpboot/ppc64le/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/httpboot/ppc64le/agent.ramdisk"', shell=True),
        ])

    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._convert_image',  autospec=True)
    @mock.patch('tripleo_common.arch.kernel_arch', autospec=True,
                return_value='x86_64')
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._get_image', autospec=True)
    def test_overcloud_create_images_with_arch_and_pltform(self,
                                                           mock_get_image,
                                                           mock_subprocess,
                                                           mock_isfile,
                                                           mock_arch,
                                                           mock_convert_image):
        mock_isfile.return_value = False
        mock_get_image.return_value = None

        parsed_args = self.check_parser(self.cmd,
                                        ['--whole-disk'],
                                        [])
        self.cmd.take_action(parsed_args)
        parsed_args = self.check_parser(self.cmd,
                                        ['--whole-disk',
                                         '--http-boot', '/httpboot/ppc64le',
                                         '--architecture', 'ppc64le'],
                                        [])
        self.cmd.take_action(parsed_args)
        parsed_args = self.check_parser(self.cmd,
                                        ['--whole-disk',
                                         '--http-boot', '/httpboot/p9-ppc64le',
                                         '--architecture', 'ppc64le',
                                         '--platform', 'p9'],
                                        [])
        self.cmd.take_action(parsed_args)

        self.assertEqual(
            0,
            self.app.client_manager.image.delete_image.call_count
        )
        self.assertEqual(
            3,
            self.app.client_manager.image.create_image.call_count
        )

        self.app.client_manager.image.create_image.assert_has_calls([
            mock.call(name='overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
            mock.call(name='ppc64le-overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
            mock.call(name='p9-ppc64le-overcloud-full',
                      disk_format='raw',
                      container_format='bare',
                      data=mock.ANY,
                      validate_checksum=False,
                      visibility='public'),
        ])

        self.app.client_manager.image.update_image.assert_has_calls([
            mock.call(10, hw_architecture='x86_64'),
            mock.call(11, hw_architecture='ppc64le'),
            mock.call(12, hw_architecture='ppc64le', tripleo_platform='p9'),
        ])
        self.assertEqual(mock_convert_image.call_count, 3)
        self.assertEqual(mock_subprocess.call_count, 6)
        mock_subprocess.assert_has_calls([
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/var/lib/ironic/httpboot/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/var/lib/ironic/httpboot/agent.ramdisk"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/httpboot/ppc64le/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/httpboot/ppc64le/agent.ramdisk"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.kernel" '
                      '"/httpboot/p9-ppc64le/agent.kernel"', shell=True),
            mock.call('sudo cp -f "./ironic-python-agent.initramfs" '
                      '"/httpboot/p9-ppc64le/agent.ramdisk"', shell=True),
        ])


class TestUploadOnlyExisting(TestPluginV1):

    def setUp(self):
        super(TestUploadOnlyExisting, self).setUp()

        # Get the command object to test
        self.cmd = overcloud_image.UploadOvercloudImage(self.app, None)
        self.app.client_manager.image = mock.Mock()
        self.app.client_manager.image.version = 2.0
        self.app.client_manager.image.create_image.return_value = (
            mock.Mock(id=10, name='imgname', properties={},
                      created_at='2015-07-31T14:37:22.000000'))
        mock_cfe = mock.patch('tripleoclient.v1.overcloud_image.'
                              'BaseClientAdapter.check_file_exists',
                              autospec=True)
        mock_cfe.start()
        self.addCleanup(mock_cfe.stop)
        mock_cfe.return_value = True

        mock_rifp = mock.patch('tripleoclient.v1.overcloud_image.'
                               'BaseClientAdapter.read_image_file_pointer',
                               autospec=True)
        mock_rifp.start()
        self.addCleanup(mock_rifp.stop)
        self._file_handle = FakeHandle()
        mock_rifp.return_value = self._file_handle

    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_try_update', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    def test_overcloud_upload_just_ipa_wholedisk(self,
                                                 mock_files_changed,
                                                 mock_image_changed,
                                                 mock_image_try_update,
                                                 mock_isfile_call,
                                                 mock_subprocess_call):
        mock_image_changed.return_value = True
        mock_image_try_update.return_value = None

        parsed_args = self.check_parser(
            self.cmd, ['--whole-disk', '--image-type=ironic-python-agent'], [])

        mock_files_changed.return_value = True
        self.cmd.take_action(parsed_args)

        # ensure check_file_exists has not been called
        self.assertCountEqual(
            self.cmd.adapter.check_file_exists.call_args_list,
            [mock.call(self.cmd.adapter, './ironic-python-agent.initramfs'),
             mock.call(self.cmd.adapter, './ironic-python-agent.kernel')])

        self.assertFalse(mock_image_try_update.called)

    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_try_update', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    def test_overcloud_upload_just_os_wholedisk(self,
                                                mock_files_changed,
                                                mock_image_changed,
                                                mock_image_try_update,
                                                mock_isfile_call,
                                                mock_subprocess_call):
        mock_image_changed.return_value = True
        mock_image_try_update.return_value = None

        parsed_args = self.check_parser(
            self.cmd, ['--whole-disk', '--image-type=os'], [])

        mock_files_changed.return_value = True
        self.cmd.take_action(parsed_args)

        # ensure check_file_exists has been called just with ipa
        self.cmd.adapter.check_file_exists.assert_called_once_with(
            self.cmd.adapter, './overcloud-full.qcow2')

        # ensure try_update has been called just with ipa
        mock_image_try_update.assert_called_once_with(
            self.cmd.adapter,
            'overcloud-full',
            './overcloud-full.raw'
        )

    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_try_update', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    def test_overcloud_upload_just_ipa(self,
                                       mock_files_changed,
                                       mock_image_changed,
                                       mock_image_try_update,
                                       mock_isfile_call,
                                       mock_subprocess_call):
        mock_image_changed.return_value = True
        mock_image_try_update.return_value = None

        parsed_args = self.check_parser(
            self.cmd, ['--image-type=ironic-python-agent'], [])

        mock_files_changed.return_value = True
        self.cmd.take_action(parsed_args)

        # ensure check_file_exists has been called just with ipa
        self.assertCountEqual(
            self.cmd.adapter.check_file_exists.call_args_list,
            [mock.call(self.cmd.adapter, './ironic-python-agent.initramfs'),
             mock.call(self.cmd.adapter, './ironic-python-agent.kernel')]
        )

        self.assertFalse(mock_image_try_update.called)

    @mock.patch('subprocess.check_call', autospec=True)
    @mock.patch('os.path.isfile', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_try_update', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'GlanceClientAdapter._image_changed', autospec=True)
    @mock.patch('tripleoclient.v1.overcloud_image.'
                'BaseClientAdapter._files_changed', autospec=True)
    def test_overcloud_upload_just_os(self,
                                      mock_files_changed,
                                      mock_image_changed,
                                      mock_image_try_update,
                                      mock_isfile_call,
                                      mock_subprocess_call):
        mock_image_changed.return_value = True
        mock_image_try_update.return_value = None

        parsed_args = self.check_parser(
            self.cmd, ['--image-type=os'], [])

        mock_files_changed.return_value = True
        self.cmd.take_action(parsed_args)

        # ensure check_file_exists has been called just with os
        self.assertCountEqual(
            self.cmd.adapter.check_file_exists.call_args_list,
            [mock.call(self.cmd.adapter, './overcloud-full.qcow2')])

        # ensure try_update has been called just with ipa
        mock_image_try_update.assert_has_calls([
            mock.call(self.cmd.adapter,
                      'overcloud-full-vmlinuz',
                      './overcloud-full.vmlinuz'),
            mock.call(self.cmd.adapter,
                      'overcloud-full-initrd',
                      './overcloud-full.initrd'),
            mock.call(self.cmd.adapter,
                      'overcloud-full',
                      './overcloud-full.raw'),
        ])
